/*
 * Copyright (c) 2010-2011 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kniss
 *
 * This file is part of de.inovat.kex.ftp.FTPKommunikation
 *
 * de.inovat.kex.ftp.FTPKommunikation is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * de.inovat.kex.ftp.FTPKommunikation is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * <p>
 * You should have received a copy of the GNU General Public License
 * along with de.inovat.kex.ftp.FTPKommunikation; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Contact Information:
 * inovat, Dipl.-Ing. H. C. Kniss
 * Koelner Strasse 30
 * D-50859 Koeln
 * +49 (0)2234 4301 800
 * info@invat.de
 * www.inovat.de
 */



package de.inovat.kex.ftp;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.sys.funclib.debug.Debug;
import org.apache.commons.net.ftp.FTPClient;

import java.io.*;

//~ JDK IMPORTE ===============================================================

//~ KLASSEN ===================================================================

/**
 * Die Klasse dient zur Verwaltung der gesamten FTP Kommunikation.
 * <p/>
 *
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @author Dagmar Rostek (DR)
 * @version $Revision$ / $Date$ / ($Author$)
 */
public class FTPKommunikation {

    /** DebugLogger fr Debug-Ausgaben. */
    private static final Debug debug = Debug.getLogger();

    //~ FELDER ================================================================

    /** FTPClient. */
    private FTPClient _ftpClient = null;

    /** ClientDavInteface. */
    private ClientDavInterface _dav;

    /** Legt fest, ob die FTP-Verbindung im PassivModus eingebaut werden soll. */
    private boolean _ftpPassivModus;

    /** Passwort zum Login auf den FTP-Server. */
    private String _passwd;

    /** Port des FTP-Servers. */
    private int _port;

    /** Temporre lokale Verzeichnis auf dem die Dateien kurz abgelebt werden. */
    private String _tempVerzeichnis;

    /** Typ des Systemobjekte fr den BASt-Band Import/Export. */
    private String _typBAStBand;

    /** Verzeichnis das berwacht wird. */
    private String _ueberwachungsVerezichnis;

    /** URL des FTP-Servers. */
    private String _url;

    /** User fr den Login auf den FTP-Server. */
    private String _user;

    /** Verzeichnis in das die Dateien nach erfolgreichem Import verschoben werden. */
    private String _verschiebeVerzeichnis;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Konstruktor.
     *
     * @param dav                      ClientDavInterface.
     * @param url                      URL des FTP-Servers.
     * @param port                     Port des FTP-Servers.
     * @param user                     User des FTP-Servers.
     * @param passwd                   Passwort fr den Login auf den FTP-Server.
     * @param ftpPassivModus           Legt fest, ob die FTP-Verbindung im PassivModus eingebaut werden soll.
     * @param typBAStBand              Typ des Systemobjekte fr den BASt-Band Import/Export.
     * @param ueberwachungsVerzeichnis das zu berwachende Verzeichnis.
     * @param verschiebeVerzeichnis    das Verzeichnis, in das die Dateien nach erfolgreichem Import verschoben werden.
     * @param tempVerzeichnis          temporre Verzeichnis.
     */
    public FTPKommunikation(ClientDavInterface dav, String url, int port, String user, String passwd, boolean ftpPassivModus, String typBAStBand, String ueberwachungsVerzeichnis, String verschiebeVerzeichnis, String tempVerzeichnis) {
        _dav                      = dav;
        _url                      = url;
        _port                     = port;
        _user                     = user;
        _passwd                   = passwd;
        _ftpPassivModus           = ftpPassivModus;
        _typBAStBand              = typBAStBand;
        _ueberwachungsVerezichnis = ueberwachungsVerzeichnis;
        _verschiebeVerzeichnis    = verschiebeVerzeichnis;
        _tempVerzeichnis          = tempVerzeichnis;

        //
        try {
            aktualisiereVerbindung();
        }
        catch (Exception e) {
            debug.warning("Die Verbindung zum FTP-Server konnte nicht erstellt werden.", e);
        }
    }

    //~ METHODEN ==============================================================

    /**
     * Die Methode aktualisiert die FTP-Verbindung.
     */
    protected void aktualisiereVerbindung() {
        if (_ftpClient != null) {
            try {
                _ftpClient.logout();
                _ftpClient.disconnect();
            }
            catch (IOException e) {
                debug.warning("Die Verbindung zum FTP-Server konnte nicht beendet werden.", e);
            }
        }

        _ftpClient = verbindeMitFTPServer();

        if (loginToFTPServer()) {
            changeDirektoryFTPServer();
        } else {
            String infoText = "Der Login auf dem FTP-Server ist gescheitert. Bitte kontrollieren Sie den angegebenen Benutzernamen und das Passwort!";

            debug.error(infoText);
        }
    }

    /**
     * Die Methode wechselt in das Exportverzeichnis fr den gestarteten Export.
     */
    public void changeDirektoryFTPServer() {
        boolean wechselOK = false;

        if ((_ftpClient != null) && (_ftpClient.isConnected())) {
            try {
                wechselZumRootFTPVerzeichnis();
                wechselOK = _ftpClient.changeWorkingDirectory(_ueberwachungsVerezichnis);
            }
            catch (IOException e) {
                String infoText = "Es konnte nicht in das per Parameter angegebene Verzeichnis " + _ueberwachungsVerezichnis + " gewechselt werden. Bitte berprfen Sie die Angaben. ";

                debug.error(infoText, e);
            }
        } else {
            aktualisiereVerbindung();
            changeDirektoryFTPServer();
        }

        if (!wechselOK) {
            String infoText = "Es konnte nicht in das per Parameter angegebene Verzeichnis " + _ueberwachungsVerezichnis + " gewechselt werden. Bitte berprfen Sie die Angaben. ";

            debug.error(infoText);
        }
    }

    /**
     * Die Methode lscht die Datei in dem berwachungsverzeichnis.
     *
     * @param dateiName Dateiname der Datei die gelscht wird.
     */
    private void deleteAlteDatei(String dateiName) {
        try {
            wechselZumRootFTPVerzeichnis();
            _ftpClient.changeWorkingDirectory(_ueberwachungsVerezichnis);
            _ftpClient.deleteFile(dateiName);
        }
        catch (IOException e) {
            debug.error("Wechsel in das berwachungsverzeichnis zum Lschen der Datei hat nicht geklappt. ", e);
        }
    }

    /**
     * Download der Datei in das Verschiebeverzeichnis.
     *
     * @param dateiName Dateiname der Datei die in das Verschiebeverzeichnis abgelegt wird.
     * @return true wenn das Speichern der Datei in dem angegebenem Verschiebeverzeichnis geklappt hat.
     */
    private boolean downloadFileImVerschiebeverzeichnis(String dateiName) {
        String  verschiebeVerzeichnis = _verschiebeVerzeichnis + dateiName;
        boolean success               = false;
        boolean fileDeleted           = false;
        File    file                  = new File(_tempVerzeichnis + dateiName);

        try {
            FileOutputStream outputStream = new FileOutputStream(file);
            InputStream      in           = _ftpClient.retrieveFileStream(dateiName);
            int              data;

            while ((data = in.read()) != -1) {
                outputStream.write(data);
            }

            aktualisiereVerbindung();

            InputStream localInput = new FileInputStream(file);

            wechselZumRootFTPVerzeichnis();

            // Sollte schon eine Datei mit gleichem Namen existieren wird diese einfach gelscht.
            _ftpClient.deleteFile(dateiName);
            success = _ftpClient.storeFile(verschiebeVerzeichnis, localInput);

            if (success) {
                outputStream.close();
                in.close();
                localInput.close();

                if (file.exists() && file.canRead() && file.canWrite()) {
                    fileDeleted = file.delete();
                } else {
                    debug.error("Die Datei konnte in dem temporren Verzeichnis nicht gelscht werden.");
                }
            }
        }
        catch (IOException e) {
            debug.error("Es sind Probleme beim Einlesen der Datei aufgetreten: ", e);
        }

        if (!fileDeleted) {
            debug.error("Die Datei konnte in dem Temporrem Verzeichnis nicht gelscht werden, bitte prfen Sie dies manuell nach.");
        }

        return success;
    }

    /**
     * Login FTP-Server.
     *
     * @return true wenn der Login geklappt hat, false sonst.
     */
    private boolean loginToFTPServer() {
        boolean loginOK = false;

        try {
            loginOK = _ftpClient.login(_user, _passwd);

            // NLStBV, HEINZE, 09.05.2012: use passive mode (every time a download/upload start)
            if (_ftpPassivModus) {
                _ftpClient.enterLocalPassiveMode();
            }
        }
        catch (IOException e) {
            String infoText = "Der login auf dem FTP-Server ist gescheitert. Bitte kontrollieren Sie den angegebenen Benutzernamen und das Passwort!";

            debug.error(infoText, e);
        }

        return loginOK;
    }

    /**
     * Die Methode zur berwachung des Verzeichnisses.
     */
    public void ueberwacheVerzeichnis() {
        if (_ftpClient.isConnected()) {
            try {

                // ---------------------------------------------------------------
                // Ermittle alle Dateien im FTP-berwachungs-Verzeichnis:
                // ---------------------------------------------------------------
                String[] alleFiles = _ftpClient.listNames();

                if (alleFiles != null) {

                    // ---------------------------------------------------------------
                    // Bearbeite alle FTP-Dateien:
                    // ---------------------------------------------------------------
                    for (String dateiName : alleFiles) {

                        // ---------------------------------------------------------------
                        // Falls Bearbeitungs-Datei --->
                        // verschiebe die in FTP-Verzeichnis _verschiebeVerzeichnis:
                        // ---------------------------------------------------------------
                        if (dateiName.endsWith(Verzeichnisueberwachung.BEARBEITUNGS_DATEI)) {
                            if (downloadFileImVerschiebeverzeichnis(dateiName)) {
                                deleteAlteDatei(dateiName);
                            }
                        }

                        // ---------------------------------------------------------------
                        // Falls nicht Bearbeitungs-Datei und nicht Lock-Datei --->
                        // Sende berwachungsdaten fr die Datei an DaV:
                        // ---------------------------------------------------------------
                        else if (!dateiName.endsWith(Verzeichnisueberwachung.LOCK_DATEI) && (!dateiName.endsWith(Verzeichnisueberwachung.BEARBEITUNGS_DATEI))) {
                            DavKommunikation davKomm = new DavKommunikation(_dav, _typBAStBand);

                            davKomm.sendeVerzeichnisUeberwachungsDaVDaten(dateiName, _ueberwachungsVerezichnis, true, _url, _port, _user, _passwd, _ftpPassivModus);
                        }
                    }
                }
            }
            catch (IOException e) {
                debug.warning("Es sind Probleme bei der FTP-Kommunikation aufgetreten.", e);
            }
        } else {
            aktualisiereVerbindung();
        }
    }

    /**
     * Die Methode stellt eine Verbindung zum FTP-Server her.
     *
     * @return FTPClient.
     */
    private FTPClient verbindeMitFTPServer() {
        if (_ftpClient == null) {
            _ftpClient = new FTPClient();
        }

        try {
            _ftpClient.connect(_url, _port);
        }
        catch (IOException e) {
            String infoText = "Es sind Probleme bei der Verbindung mit dem FTP-Server aufgetreten.";

            debug.error(infoText, e);

            return null;
        }

        return _ftpClient;
    }

    /**
     * Die Methode wechselt ins Root Verzeichnis des FTP-Servers.
     */
    private void wechselZumRootFTPVerzeichnis() {
        try {
            _ftpClient.changeToParentDirectory();
            _ftpClient.changeToParentDirectory();
            _ftpClient.changeToParentDirectory();
        }
        catch (IOException e) {
            debug.error("Wechsel ins Root Verzeichnis des FTP-Servers hat nicht geklappt.", e);
        }
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 27.06.13
