/*
 * Copyright (c) 2010-2011 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kniss
 *
 * This file is part of de.inovat.kex.ftp.Verzeichnisueberwachung
 *
 * de.inovat.kex.ftp.Verzeichnisueberwachung is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * de.inovat.kex.ftp.Verzeichnisueberwachung is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * <p>
 * You should have received a copy of the GNU General Public License
 * along with de.inovat.kex.ftp.Verzeichnisueberwachung; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Contact Information:
 * inovat, Dipl.-Ing. H. C. Kniss
 * Koelner Strasse 30
 * D-50859 Koeln
 * +49 (0)2234 4301 800
 * info@invat.de
 * www.inovat.de
 */



package de.inovat.kex.ftp;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.sys.funclib.application.StandardApplication;
import de.bsvrz.sys.funclib.application.StandardApplicationRunner;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;
import de.bsvrz.sys.funclib.debug.Debug;

//~ KLASSEN ===================================================================

/**
 * Die Klasse stellt alle Methoden zur Verfgung, die das via parameter bergebene Verzeichnis bzgl. Importdateien
 * berprft.
 * <p/>
 *
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @author Dagmar Rostek (DR)
 * @version $Revision$ / $Date$ / ($Author$)
 */
public class Verzeichnisueberwachung implements Runnable, StandardApplication {

    /** Datei in der Bearbeitung. */
    protected static final String BEARBEITUNGS_DATEI = ".bearbeitet";

    /** Lock-Datei. */
    protected static final String LOCK_DATEI = ".lock";

    /** DebugLogger fr Debug-Ausgaben */
    private static Debug debug = Debug.getLogger();

    /**
     * Boolesche Attribut das den Threadzustand angibt. Wenn _laufeThread == true ist, luft der Thread, sonst ist das
     * Attribut = false.
     */
    private static boolean _laufeThread;

    /** Thread zur Prfung des Verzeichnisses. */
    private static Thread _threadVerzeichnisUeberwachung;

    //~ FELDER ================================================================

    /** Instanz der Klasse FTPKommunikation. */
    private FTPKommunikation _ftpKommunikation;

    /** Legt fest, ob die FTP-Verbindung im PassivModus eingebaut werden soll. */
    private boolean _ftpPassivModus;

    /**
     * Definiert den ServerTyp: FTP oder SFTP:
     * falls true - FTP,
     * sonst - SFTP.
     */
    private boolean _istFtp;

    /** Attribut dass das loginpasswort fr den FTP-Server angibt. */
    private String _passwd;

    /** Attribut das den Portt angibt. */
    private int _port;

    /**
     * Integer Attribut, dass die Zeit in Sekunden angibt, wie hufig das bergebene Verzeichnis auf Dateien berprft
     * werden soll.
     */
    private int _pruefZeitRaum;

    /** Instanz der Klasse SFTPKommunikation. */
    private SFTPKommunikation _sftpKommunikation;

    /** Temporre Verzeichnis, in das die zu Importierenden Dateien temporr gespeichert werden. */
    private String _tempVerzeichnis;

    /** Typ des Systemobjekte fr den BASt-Band Import/Export. */
    private String _typBAStBand;

    /** Attribut das den host angibt. */
    private String _url;

    /** Attribut dass den login user fr den FTP-Server angibt. */
    private String _user;

    /** Attribut das den Pfad zum Verschiebverzeichnis angibt. */
    private String _verschiebeVerzeichnis;

    /** Attribut das den Pfad zum zu berwachenden Verzeichnis angibt. */
    private String _zuUeberwachendeVerzeichnis;

    //~ METHODEN ==============================================================

    /**
     * Diese Methode wird nach dem Verbindungsaufbau zum Datenverteiler aufgerufen. Die Applikation bekommt damit die
     * Mglichkeit sich zu initialisieren. Es wird eine Instanz FTPKommunikation erzeugt und mit den Argumenten
     * initialisiert. Wenn das Prfverzeichnis wie auch das Verschiebeverzeichnis zugnglich ist wird der Thread zur
     * Verzeichnisberwachung gestartet. Wenn nicht wird eine Fehlermeldung ausgegeben.
     *
     * @param dav Verbindung zum Datenverteiler
     *
     * @throws Exception Falls ein unerwarteter _Fehler aufgetreten ist.
     */

    // @Override
    public void initialize(ClientDavInterface dav) throws Exception {

        // FTP:
        if (_istFtp) {
            _ftpKommunikation = new FTPKommunikation(dav, _url, _port, _user, _passwd, _ftpPassivModus, _typBAStBand, _zuUeberwachendeVerzeichnis, _verschiebeVerzeichnis, _tempVerzeichnis);
        }

        // SFTP:
        else {
            _sftpKommunikation = new SFTPKommunikation(dav, _url, _port, _user, _passwd, _typBAStBand, _zuUeberwachendeVerzeichnis, _verschiebeVerzeichnis);
        }

        starteThread();
    }

    /**
     * Main-Methode.
     *
     * @param args Aufrufargumente.
     */
    public static void main(String args[]) {
        StandardApplicationRunner.run(new Verzeichnisueberwachung(), args);
        debug = Debug.getLogger();
    }

    // @Override
    public void parseArguments(ArgumentList argumentList) throws Exception {
        debug = Debug.getLogger();

        boolean istOk = true;
        String  par;

        // --------------------------------------------------------------------
        // Hole den Parameter fr FTP/SFTP:
        par = "-istFtp";

        try {
            _istFtp = argumentList.fetchArgument(par).booleanValue();
        }
        catch (Exception e) {
            _istFtp = true;
        }

        // --------------------------------------------------------------------
        // Hole den Parameter fr Host:
        par = "-ftpServerURL";

        try {
            _url = argumentList.fetchArgument(par).asNonEmptyString();
        }
        catch (Exception e) {
            debug.error(String.format("Bitte Host als Parameter <%s> bergeben!", par), e);
            istOk = false;
        }

        // --------------------------------------------------------------------
        // Hole den Parameter fr Server-Port:
        par = "-ftpServerPort";

        try {
            _port = argumentList.fetchArgument(par).intValue();
        }
        catch (Exception e) {
            if (_istFtp) {
                _port = 21;
            } else {
                _port = 22;
            }
        }

        // --------------------------------------------------------------------
        // Hole den Parameter fr Server-Benutzer:
        par = "-ftpServerBenutzer";

        try {
            _user = argumentList.fetchArgument(par).asNonEmptyString();
        }
        catch (Exception e) {
            debug.error(String.format("Bitte Server-Benutzer als Parameter <%s> bergeben!", par), e);
            istOk = false;
        }

        // --------------------------------------------------------------------
        // Hole den Parameter fr Server-Passwort:
        par = "-ftpServerPasswort";

        try {
            _passwd = argumentList.fetchArgument(par).asNonEmptyString();
        }
        catch (Exception e) {
            debug.error(String.format("Bitte Server-Passwort als Parameter <%s> bergeben!", par), e);
            istOk = false;
        }

        // --------------------------------------------------------------------
        // Hole den Parameter fr PassivModus fr FTP-Server:
        par = "-ftpPassivModus";

        try {
            _ftpPassivModus = argumentList.fetchArgument(par).booleanValue();
        }
        catch (Exception e) {
            _ftpPassivModus = false;
        }

        // --------------------------------------------------------------------
        // Hole den Parameter fr den zu berwachenden Verzeichnis:
        par = "-verzeichnisUeberwachung";

        try {
            _zuUeberwachendeVerzeichnis = argumentList.fetchArgument(par).asNonEmptyString();
        }
        catch (Exception e) {
            debug.error(String.format("Bitte der zu berwachende Verzeichnis als Parameter <%s> bergeben!", par), e);
            istOk = false;
        }

        // --------------------------------------------------------------------
        // Hole den Parameter fr den Verschieben-Verzeichnis:
        par = "-verschiebeVerzeichnis";

        try {
            _verschiebeVerzeichnis = argumentList.fetchArgument(par).asNonEmptyString();
        }
        catch (Exception e) {
            debug.error(String.format("Bitte der Verschieben-Verzeichnis als Parameter <%s> bergeben!", par), e);
            istOk = false;
        }

        // --------------------------------------------------------------------
        // Hole den Parameter fr die berwachungszeit in Sekunden:
        par = "-pruefZeitraum";

        try {
            _pruefZeitRaum = argumentList.fetchArgument(par).intValue();
        }
        catch (Exception e) {
            debug.error(String.format("Bitte die berwachungszeit in Sekunden als Parameter <%s> bergeben!", par), e);
            istOk = false;
        }

        // --------------------------------------------------------------------
        // Hole den Parameter fr den temporelen Verzeichnis:
        par = "-tempVerzeichnis";

        try {
            _tempVerzeichnis = argumentList.fetchArgument(par).asNonEmptyString();
        }
        catch (Exception e) {
            debug.error(String.format("Bitte das temporaere Verzeichnis als Parameter <%s> bergeben!", par), e);
            istOk = false;
        }

        // --------------------------------------------------------------------
        // Hole den Parameter fr die BASt-Export/Import-Steuerung:
        par = "-pidBastExportImportSteuerung";

        try {
            _typBAStBand = argumentList.fetchArgument(par).asNonEmptyString();
        }
        catch (Exception e) {
            debug.error(String.format("Bitte die BASt-Export/Import-Steuerung als Parameter <%s> bergeben!", par), e);
            istOk = false;
        }

        // --------------------------------------------------------------------
        if (!istOk) {
            stopThread("Die Anwendung wird geschlossen, da die Parameter nicht den notwendigen Parametern entsprechen.");
            System.exit(0);
        }
    }

    // @Override
    public void run() {
        while (_laufeThread) {
            try {
                Thread.sleep(_pruefZeitRaum * 1000);
            }
            catch (InterruptedException e) {
                debug.warning("Es sind Probleme mit dem Thread 'Pause' aufgetreten.", e);
            }

            try {

                // FTP:
                if (_istFtp) {
                    _ftpKommunikation.changeDirektoryFTPServer();
                    _ftpKommunikation.aktualisiereVerbindung();
                    _ftpKommunikation.ueberwacheVerzeichnis();
                }

                // SFTP:
                else {
                    _sftpKommunikation.ueberwacheVerzeichnis();
                }
            }
            catch (Exception e) {
                debug.warning("Es sind Probleme im Prozess 'Verzeichnisberwachung' aufgetreten.", e);
            }
        }
    }

    /**
     * Die Methode erzeugt einen Thread, der die Verzeichnisberwachung startet. Durch die Verwendung eines Threads ist
     * es mglich den Export zu jedem Zeitpunkt zu unterbrechen.
     */
    private void starteThread() {
        if (!_laufeThread) {
            _threadVerzeichnisUeberwachung = new Thread(this);
            _threadVerzeichnisUeberwachung.setName("VerzeichnisUeberwachung");
            _laufeThread = true;
            _threadVerzeichnisUeberwachung.start();
        }
    }

    /**
     * Stoppt den Thread.
     *
     * @param meldung Statusmeldung die an den Clienten via DaV geschickt wird.
     */
    public void stopThread(String meldung) {
        if (_laufeThread) {

            // noinspection AssignmentToNull
            _laufeThread = false;
        }

        debug.info("Der Thread wurde beendet da: " + meldung);
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 27.06.13
